import os
import numpy as np
import math
import six
import iris
import iris.cube
from iris.cube import CubeList
import iris.analysis
import iris.util
import iris.coords
import warnings
import numba
import datetime
from numba import vectorize
from numba import cuda
from numba import int32, int64, float32, float64
from numba.cuda.cudadrv import driver

MESH_FILE = "/esarchive/autosubmit/con_files/mesh_mask_nemo.Ec3.2_O1L75.nc"
REGIONS_FILE = "/esarchive/autosubmit/con_files/mask.regions.Ec3.2_O1L75.nc"
VO_FILE = '/esarchive/exp/ecearth/a16l/cmorfiles/CMIP/EC-Earth-Consortium' \
          '/EC-Earth3-LR/historical/r1i1p1f1/Omon/vo/gn/v20180711' \
          '/vo_Omon_EC-Earth3-LR_historical_r1i1p1f1_gn_196101-196112.nc'


def main():
    e3t, e1v = load_mesh()
    basin_name, basins = load_masks()
    area = {}
    for basin in basins:
        area[basin] = compute_area_basin(e1v, e3t, basins[basin])
    del e3t, e1v
    vo = load_vo()
    vsftmyx = compute_vsftmyx_cpu(vo, area)
    a=2




def compute_vsftmyx_cpu(vo, area):
    vsftmyx = []
   # for basin in area:
    vsftmyx_basin = np.sum(multiply_vo_basin(vo, area['Global_Ocean']), axis=3)
    cube = iris.cube.Cube(vsftmyx_basin)
    cube
    iris.save(cube, '/esarchive/scratch/sloosvel/numba_outputs/vsftmyx.nc', zlib=True)
    vsftmyx.append(vsftmyx_basin)
    return vsftmyx




def load_vo():
    vo = iris.load_cube(VO_FILE, 'sea_water_y_velocity')
    vo_data = np.ma.filled(vo.data, 0.0).astype(np.float32)
    return vo_data


def load_mesh():
    e3t = iris.load_cube(MESH_FILE, 'e3t_0').data.astype(np.float32)
    e3t = np.squeeze(e3t)
    e1v = iris.load_cube(MESH_FILE, 'e1v').data.astype(np.float32)
    e1v = np.squeeze(e1v)
    return e3t, e1v


def load_masks():
    basins = {}
    cubes = iris.load(REGIONS_FILE)
    basin_name = []
    for cube in cubes:
        name = cube.name()
        if name in ('nav_lat', 'nav_lon', 'Caspian_Sea'):
            continue
        basin_name.append(name)
        cube = cube.extract(iris.Constraint(z=1))
        basins[name] = cube.data.astype(np.float32)
    return basin_name, basins


@vectorize(['float32(float32, float32, float32)'], target='cpu')
def compute_area_basin(e1v, e3t, basin):
    return - e1v *e3t * basin / 1e6


@vectorize(['float32(float32, float32)'], target='cpu')
def multiply_vo_basin(vo, basin):
    return vo*basin


@vectorize(['float64(float32, float32)', 'float64(float64, float64)'],
           target='cpu')
def sum_red(x, y):
    return x+y


if __name__ == '__main__':
    main()